/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file inpaint.cpp
@ingroup ExamplePrograms
@brief Does inpaint reconstruction over artificialy damaged frames from a video input source.

This program will receive a path or url to a video file, and then will show the reconstruction of every frame of that video,
previously eliminating a fixed set of pixels from it.
The file mask.gif contains a grayscale image, where pixels with a value different from zero will be erased from every frame
of the given video. Those frames are later reconstructed with inpainting functionality of the QVIPP package, before being
shown to the user.

@section Usage
Simply compile, and execute the binary:
\code
	./inpaint --URL=<uri for input video> --Mask=<mask image file name>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@param Mask Name for the image mask file.
@author PARP Research Group. University of Murcia, Spain.
*/
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>
#include <qvipp/qvipp.h>

/// \cond
class MyWorker: public QVWorker
	{
	private:
		QVImage<uChar> mask;

	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty<QString>("Mask", inputFlag,	"", "Image file for mask");
			addProperty<int>("Radius", inputFlag, 4, "Mask radius", 1, 30);
			addProperty<bool>("Use Telea", inputFlag, false, "Use Telea versus NS");
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Mask image", outputFlag);
			addProperty< QVImage<uChar,3> >("Restored image", outputFlag);
		
			// Here we try to open mask file. If it can't be opened, we finish.
			if(!QVMPlayerCamera::getFrame(getPropertyValue<QString>("Mask"), mask))
				setLastError("Error, can't find mask image file");
			}

		void iterate()
			{
			bool useTelea =	getPropertyValue<bool>("Use Telea");
			int radius = getPropertyValue<int>("Radius");
		
			QVImage<uChar,3> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			timeFlag("init");
		
			///////////
			// Obtain mask image of size equal to image
			QVImage<uChar> maskForImage(image.getCols(), image.getRows());
			Resize(mask, maskForImage);
		
			// ESTO NO TENDRíA QUE SER NECESARIO.
			for (uInt col = 0; col < image.getCols(); col++)
				for (uInt row = 0; row < image.getRows(); row++)
					if (maskForImage(col, row) < 128)
						maskForImage(col, row) = 0;
					else
						maskForImage(col, row) = 255;
		
			timeFlag("Obtain mask image of size equal to image");
		
			////////////
			// Get distances using fast marching algorithm
			QVImage<uChar> buffer;
			FastMarchingGetBufferSize(maskForImage, buffer);
		
			QVImage<sFloat> distances(image.getCols(), image.getRows());
			Set(distances,0);
			FastMarching(maskForImage, distances, radius, buffer);
			timeFlag("Get distances using fast marching algorithm");
		
			/////////////
			// Inpainting
			QVImage<uChar,3> inpaint(image.getCols(),image.getRows());
			Inpaint(image, maskForImage, distances, inpaint, radius, useTelea?IPP_INPAINT_TELEA:IPP_INPAINT_NS);
			timeFlag("Inpainting");
						
			/////////////
			// Showing results
			setPropertyValue< QVImage<uChar,1> >("Mask image", maskForImage);
			setPropertyValue< QVImage<uChar,3> >("Restored image", inpaint);
			timeFlag("Showing results");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Does inpaint reconstruction from an artificialy damaged source video.");

	QVMPlayerCamera camera("Video");
	MyWorker worker("Inpaint worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas inputImage("Input image");
	inputImage.linkProperty(worker,"Input image");

	QVImageCanvas maskImage("Mask image");
	maskImage.linkProperty(worker,"Mask image");

	QVImageCanvas restoredImage("Restored image");
	restoredImage.linkProperty(worker,"Restored image");

	return app.exec();
	}

/// \endcond

